import sys

from . import __version__
from .cli import get_args, get_code
from .carbon import create_code_image


def main():
    args = get_args()
    if args.version:
        print(__version__)
        sys.exit(0)

    code = get_code(args)
    options = {
        "language": args.language,
        "background": args.background,
        "theme": args.theme,
        "interactive": args.interactive,
        "destination": args.destination,
    }
    create_code_image(code, **options)


if __name__ == "__main__":
    main()



import argparse
import os

import pyperclip


def get_args():
    parser = argparse.ArgumentParser(description="Create a carbon code image")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("-f", "--file", type=str, help="File with code")
    group.add_argument(
        "-c", "--clipboard", action="store_true", help="Use code on clipboard"
    )
    group.add_argument("-s", "--snippet", type=str, help="Code snippet")
    group.add_argument(
        "-v", "--version", action="store_true", default=False, help="Show version"
    )
    parser.add_argument(
        "-i",
        "--interactive",
        action="store_true",
        default=False,
        help="Run Selenium in interactive (not headless) mode",
    )
    parser.add_argument("-l", "--language", type=str, help="Programming language")
    parser.add_argument("-b", "--background", type=str, help="Background color")
    parser.add_argument("-t", "--theme", type=str, help="Name of the theme")
    parser.add_argument(
        "-d",
        "--destination",
        type=str,
        default=os.getcwd(),
        help="Specify folder where image should be stored (defaults to current directory)",
    )
    return parser.parse_args()


def get_code(args: argparse.Namespace) -> str:
    """
    Argparse's add_mutually_exclusive_group already guaranteers
    we get one of clipboard, file or snippet
    """
    if args.clipboard:
        code = pyperclip.paste()
    elif args.file:
        with open(args.file) as f:
            code = f.read()
    else:
        code = args.snippet

    return code



import os
from time import sleep
from urllib.parse import quote_plus

from dotenv import load_dotenv
from selenium import webdriver
from selenium.webdriver.chrome.options import Options

load_dotenv()

CARBON_URL = "https://carbon.now.sh?l={language}&code={code}&bg={background}&t={theme}"
CHROMEDRIVER_PATH = os.environ["CHROMEDRIVER_PATH"]

# in case of a slow connection it might take a bit longer to download the image
SECONDS_SLEEP_BEFORE_DOWNLOAD = int(os.environ.get("SECONDS_SLEEP_BEFORE_DOWNLOAD", 3))

DEFAULT_LANGUAGE = "python"
DEFAULT_BACKGROUND = "#ABB8C3"
DEFAULT_THEME = "seti"


def _create_carbon_url(code, **kwargs: str) -> str:
    language = kwargs.get("language") or DEFAULT_LANGUAGE
    background = kwargs.get("background") or DEFAULT_BACKGROUND
    theme = kwargs.get("theme") or DEFAULT_THEME

    url = CARBON_URL.format(
        language=quote_plus(language),
        code=quote_plus(code),
        background=quote_plus(background),
        theme=quote_plus(theme),
    )

    return url


def create_code_image(code: str, **kwargs: str) -> None:
    """Generate a beautiful Carbon code image"""
    options = Options()
    options.headless = not bool(kwargs.get("interactive", False))
    destination = kwargs.get("destination", os.getcwd())
    prefs = {"download.default_directory": destination}
    options.add_experimental_option("prefs", prefs)

    url = _create_carbon_url(code, **kwargs)
    with webdriver.Chrome(CHROMEDRIVER_PATH, options=options) as driver:
        driver.get(url)
        driver.find_element_by_id("export-menu").click()
        driver.find_element_by_id("export-png").click()
        # make sure it has time to download the image
        sleep(SECONDS_SLEEP_BEFORE_DOWNLOAD)



from pathlib import Path

import pytest
import pytesseract

from carbon.carbon import _create_carbon_url, create_code_image

CARBON_DOWNLOAD_FILE = Path("carbon.png")
ONE_LINE_SNIPPET = """print('hello world')"""
LONGER_CODE_SNIPPET = """
@pytest.fixture(scope="session")
def driver():
    driver = webdriver.Chrome()
    try:
        driver.get("http://localhost:8000/")
        yield driver
    except WebDriverException:
        raise RuntimeError("Cannot get to localhost:8000, did you start FastAPI?")
    finally:
        driver.quit()
"""


def test_create_image_for_one_liner():
    create_code_image(ONE_LINE_SNIPPET)
    assert CARBON_DOWNLOAD_FILE.exists()
    image_text = pytesseract.image_to_string(CARBON_DOWNLOAD_FILE.name)
    assert "hello world" in image_text


def test_create_image_for_larger_snippet():
    create_code_image(LONGER_CODE_SNIPPET)
    assert CARBON_DOWNLOAD_FILE.exists()
    image_text = pytesseract.image_to_string(CARBON_DOWNLOAD_FILE.name)
    # not getting full text, but at least some snippets which show it worked
    assert "fixture" in image_text
    assert "Chrome" in image_text
    assert "yield" in image_text
    assert "except" in image_text


def test_storing_image_in_different_folder(tmpdir):
    carbon_file = tmpdir / CARBON_DOWNLOAD_FILE
    if carbon_file.exists():
        carbon_file.unlink()
    assert not carbon_file.exists()
    create_code_image(ONE_LINE_SNIPPET, destination=tmpdir.strpath)
    assert carbon_file.exists()


@pytest.mark.parametrize(
    "code, kwargs, expected",
    [
        ("hello", {}, "https://carbon.now.sh?l=python&code=hello&bg=%23ABB8C3&t=seti"),
        (
            "hello",
            {"language": "javascript"},
            "https://carbon.now.sh?l=javascript&code=hello&bg=%23ABB8C3&t=seti",
        ),
        (
            "hello world",
            {},
            "https://carbon.now.sh?l=python&code=hello+world&bg=%23ABB8C3&t=seti",
        ),
        (
            "print('hello world')",
            {},
            "https://carbon.now.sh?l=python&code=print%28%27hello+world%27%29&bg=%23ABB8C3&t=seti",
        ),
        (
            "hello",
            {"theme": "material"},
            "https://carbon.now.sh?l=python&code=hello&bg=%23ABB8C3&t=material",
        ),
        (
            "hello",
            {"background": "#C4F2FD"},
            "https://carbon.now.sh?l=python&code=hello&bg=%23C4F2FD&t=seti",
        ),
        (
            "hello",
            {"background": "#D7FFC5", "theme": "text"},
            "https://carbon.now.sh?l=python&code=hello&bg=%23D7FFC5&t=text",
        ),
    ],
)
def test_create_carbon_url(code, kwargs, expected):
    assert _create_carbon_url(code, **kwargs) == expected



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['carbon']

package_data = \
{'': ['*']}

install_requires = \
['pyperclip >=1.8.2', 'python-dotenv >=0.19.0', 'selenium >=3.141.0']

extras_require = \
{'test': ['pytest', 'mypy', 'flake8']}

entry_points = \
{'console_scripts': ['carbon = carbon.__main__:main']}

setup(name='pybites-carbon',
      version='0.3',
      description='A package to automate making beautiful code snippets using Carbon',
      author=None,
      author_email='Bob Belderbos <bob@pybit.es>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
